@@ -28,11 +28,11 @@
 
 #include <glib.h>
 #include <gusb.h>
-#include <colord.h>
 
 /* device constants */
-#define	CH_USB_VID				0x04d8
-#define	CH_USB_PID				0xf8da
+#define	CH_USB_VID				0x273f
+#define	CH_USB_PID_BOOTLADER			0x1000
+#define	CH_USB_PID_FIRMWARE			0x1001
 #define	CH_USB_CONFIG				0x0001
 #define	CH_USB_INTERFACE			0x0000
 #define	CH_USB_HID_EP				0x0001
@@ -40,6 +40,10 @@
 #define	CH_USB_HID_EP_OUT			(CH_USB_HID_EP | 0x00)
 #define	CH_USB_HID_EP_SIZE			64
 
+/* old device constants */
+#define	CH_USB_VID_LEGACY			0x04d8
+#define	CH_USB_PID_LEGACY			0xf8da
+
 /* the default timeout in client tools */
 #define CH_DEVICE_USB_TIMEOUT			5000 /* ms */
 
@@ -316,12 +320,12 @@
  * CH_CMD_TAKE_READINGS:
  *
  * Take a reading taking into account just dark offsets.
- * All of @red, @green and @blue are _signed_ values.
+ * All of @red, @green and @blue are packed float values.
  *
  * This command is useful if you want to do an ambient reading.
  *
  * IN:  [1:cmd]
- * OUT: [1:retval][1:cmd][2:red][2:green][2:blue]
+ * OUT: [1:retval][1:cmd][4:red][4:green][4:blue]
  *
  * This command is only available in firmware mode.
  **/
@@ -436,10 +440,10 @@
 /**
  * CH_CMD_GET_PRE_SCALE:
  *
- * Get the pre XYZ scaling constant.
+ * Get the pre XYZ scaling constant. @scale is a packed float.
  *
  * IN:  [1:cmd]
- * OUT: [1:retval][1:cmd][2:scale]
+ * OUT: [1:retval][1:cmd][4:scale]
  *
  * This command is only available in firmware mode.
  **/
@@ -448,9 +452,9 @@
 /**
  * CH_CMD_SET_PRE_SCALE:
  *
- * Set the pre XYZ scaling constant.
+ * Set the pre XYZ scaling constant. @scale is a packed float.
  *
- * IN:  [1:cmd][2:scale]
+ * IN:  [1:cmd][4:scale]
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in firmware mode.
@@ -460,10 +464,10 @@
 /**
  * CH_CMD_GET_POST_SCALE:
  *
- * Get the post XYZ scaling constant.
+ * Get the post XYZ scaling constant. @scale is a packed float.
  *
  * IN:  [1:cmd]
- * OUT: [1:retval][1:cmd][2:scale]
+ * OUT: [1:retval][1:cmd][4:scale]
  *
  * This command is only available in firmware mode.
  **/
@@ -472,9 +476,9 @@
 /**
  * CH_CMD_SET_POST_SCALE:
  *
- * Set the post XYZ scaling constant.
+ * Set the post XYZ scaling constant. @scale is a packed float.
  *
- * IN:  [1:cmd][2:scale]
+ * IN:  [1:cmd][4:scale]
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in firmware mode.
@@ -585,6 +589,46 @@
  **/
 #define	CH_CMD_GET_PCB_ERRATA			0x33
 
+/**
+ * CH_CMD_SET_REMOTE_HASH:
+ *
+ * Sets the remote SHA1 hash of the profile. This is designed to
+ * be used by the calibration program to indicate the key which allows
+ * the completed profile to be found from a public web service.
+ *
+ * IN:  [1:cmd][20:sha1_hash]
+ * OUT: [1:retval][1:cmd]
+ *
+ * This command is available in firmware mode.
+ **/
+#define	CH_CMD_SET_REMOTE_HASH			0x34
+
+/**
+ * CH_CMD_GET_REMOTE_HASH:
+ *
+ * Gets the remote hash which is used to get the last profile saved
+ * to a public web service.
+ *
+ * IN:  [1:cmd]
+ * OUT: [1:retval][1:cmd][20:sha1_hash]
+ *
+ * This command is available in firmware mode.
+ **/
+#define	CH_CMD_GET_REMOTE_HASH			0x35
+
+/**
+ * CH_CMD_SELF_TEST:
+ *
+ * Tests the device by trying to get a non-zero reading from each
+ * color channel.
+ *
+ * IN:  [1:cmd]
+ * OUT: [1:retval][1:cmd]
+ *
+ * This command is available in bootloader and firmware mode.
+ **/
+#define	CH_CMD_SELF_TEST			0x40
+
 /* secret code */
 #define	CH_WRITE_EEPROM_MAGIC			"Un1c0rn2"
 
@@ -672,6 +716,12 @@ typedef enum {
 	CH_ERROR_OVERFLOW_STACK,
 	CH_ERROR_DEVICE_DEACTIVATED,
 	CH_ERROR_INCOMPLETE_REQUEST,
+	CH_ERROR_SELF_TEST_SENSOR,
+	CH_ERROR_SELF_TEST_RED,
+	CH_ERROR_SELF_TEST_GREEN,
+	CH_ERROR_SELF_TEST_BLUE,
+	CH_ERROR_SELF_TEST_COLOR_SELECT,
+	CH_ERROR_SELF_TEST_MULTIPLIER,
 	CH_ERROR_LAST
 } ChError;
 
